/*
 * Copyright 2008 IGM/LRI,  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 *  This file is part of SynteView.
 *  SynteView is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  SynteView is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with SynteView.  If not, see <http://www.gnu.org/licenses/>.
 *  
 * Please contact Frederic LEMOINE (frederic.lemoine@lri.fr ) if you need 
 * additional information or have any questions.
 * 
 */

package org.synteny.factory;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.exceptions.NotRightTypeException;
import org.synteny.Comparison;
import org.synteny.ComparisonImpl;
import org.synteny.Genome;
import org.synteny.Ortholog;
import org.synteny.OrthologImpl;
import org.synteny.Paralog;
import org.synteny.ParalogImpl;
import org.synteny.Protein;
import org.synteny.ProteinImpl;
import org.synteny.SpeciesTree;
import org.synteny.ihm.preferences.Settings;
import org.synteny.search.ProteinInfoSearch;



public class DatabaseComparisonFactory implements ComparisonsFactory {

	Collection<ComparisonFactoryListener> _listeners;
	ProteinInfoSearchFactory _infosFactory;
	SpeciesTree _speciesTree;
	Genome _refGenome;
	Collection<Genome> _compGenomes;

	public DatabaseComparisonFactory(){
		try {
			Class.forName("org.postgresql.Driver");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		_listeners = new ArrayList<ComparisonFactoryListener>();
		_infosFactory = new DatabaseProteinInfoSearchFactory();
		_compGenomes = new ArrayList<Genome>();
		databaseManager = new DatabaseManagerImpl();
	}
	
	public void addComparisonFactoryListener(
			ComparisonFactoryListener comparisonFactoryListener) {
		_listeners.add(comparisonFactoryListener);
	}

	public Collection getComparisonFactoryListener() {
		return _listeners;
	}

	public ProteinInfoSearchFactory getProteinInfoSearchFactory() {
		return _infosFactory;
	}

	public List newComparisonsList(SpeciesTree spt, Genome refGenome,Collection<Genome> compGenomes) {
		_speciesTree = spt;
		_refGenome = _speciesTree.getGenome(refGenome.getSpeciesAbbrv());
		_compGenomes.clear();
		_compGenomes.addAll(compGenomes);
		List<Comparison> lc = getComparisons(); 
		return lc;
	}

	private List<Comparison> getComparisons(){
		List<Comparison> existingComp = new ArrayList<Comparison>();
		Paralog previousParalog=null;
		Comparison previousComparison = null;
		ProteinInfoSearch searcher = _infosFactory.createProteinSearch();
		int currentBlocId = -1;
		Paralog paraCourant = null;
		Comparison comparison = null;
		Genome compGenome = null;
		Genome previousCompGenome = null;
		
		try {
			DatabaseQueryResult dbqr = sendQuery();
			for(Vector line : (Vector<Vector>)(dbqr.getDataVector())){
				int blocid = getInteger(line.elementAt(0));
				String compSpeciesAbbrv = (String)(line.elementAt(2));
				int pidSp1 = getInteger(line.elementAt(3));
				int pidSp2 = getInteger(line.elementAt(8));
				String geneNameSp1 = (String)(line.elementAt(7));
				String geneNameSp2 = (String)(line.elementAt(12));
				boolean strandSp1 = ((line.elementAt(4)).equals("+"));
				boolean strandSp2 = ((line.elementAt(9)).equals("+"));
				int startSp1 = getInteger(line.elementAt(5));
				int stopSp1  = getInteger(line.elementAt(6));

				if(comparison==null || !(previousCompGenome.getSpeciesAbbrv().equals(compSpeciesAbbrv))){
					compGenome = _speciesTree.getGenome(compSpeciesAbbrv);
					previousCompGenome = compGenome;
					for(ComparisonFactoryListener cfl : _listeners){
						cfl.genomeAnalysed(compGenome.getSpeciesName(),108);
					}
					comparison = new ComparisonImpl(compGenome,_refGenome);
					compGenome.setCompComparison(comparison);
					_refGenome.addRefComparison(comparison);
					existingComp.add(comparison);
				}

				Protein p1 = _refGenome.getProtein(pidSp1);
				Protein p2 = compGenome.getProtein(pidSp2);

				if(p1==null){
					p1 = new ProteinImpl(pidSp1);
					p1.setStrand(strandSp1);
					p1.setGenome(_refGenome);
					p1.setName(geneNameSp1);
					p1.setGenomeStartStop(startSp1, stopSp1);
					p1.setProteinInfoSearch(searcher);
					_refGenome.addProtein(p1);
				}
				if(p2==null){
					p2 = new ProteinImpl(pidSp2);
					p2.setStrand(strandSp2);
					p2.setGenome(compGenome);
					p2.setName(geneNameSp2);
					p2.setProteinInfoSearch(searcher);
					compGenome.addProtein(p2);
				}
				p1.addMatchInComparison(compGenome);
				if(paraCourant==null || blocid != currentBlocId){
					paraCourant = new ParalogImpl(blocid);
					currentBlocId = blocid;
					if(previousParalog!=null){
						//previousParalog
						previousComparison.addParalog(previousParalog);
					}
					previousParalog = paraCourant;
					previousComparison = comparison;
				}
				Ortholog orthoCourant = new OrthologImpl(blocid,paraCourant);
				orthoCourant.setCompProtein(p2);
				orthoCourant.setRefProtein(p1);
				p2.addOrtholog(orthoCourant);
				p1.addOrtholog(orthoCourant);
				paraCourant.addOrtholog(orthoCourant);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return existingComp;
	}
	
	public void setComparisonFactoryListener(
			Collection comparisonFactoryListener) {
		_listeners = comparisonFactoryListener;
	}

	public void setProteinInfoSearchFactory(
			ProteinInfoSearchFactory proteinInfoSearchFactory) {
		_infosFactory = proteinInfoSearchFactory;
	}
	
	public DatabaseQueryResult sendQuery() throws Exception{
		String sptab = "";
		Iterator<Genome> i = _compGenomes.iterator();
		while(i.hasNext()){
			sptab+="\'"+i.next().getSpeciesAbbrv()+"\'";
			if(i.hasNext()){
				sptab+=",";
			}
		}
		String query = Settings.getInstance().getSyntenyQuery().replace("%spabrv", _refGenome.getSpeciesAbbrv()).replace("%sptab", sptab);
		System.out.println("\n\n"+query+"\n\n");
		for(Object o: getComparisonFactoryListener()){
			ComparisonFactoryListener l = (ComparisonFactoryListener)o;
			l.genomeAnalysed("Querying the database", 100);
		}
		return getDatabaseManager().sendQuery(query);
	}

	/**
	 * @uml.property  name="databaseManager"
	 * @uml.associationEnd  inverse="databaseComparisonFactory:org.synteny.factory.DatabaseManager"
	 */
	private DatabaseManager databaseManager;

	/**
	 * Getter of the property <tt>databaseManager</tt>
	 * @return  Returns the databaseManager.
	 * @uml.property  name="databaseManager"
	 */
	public DatabaseManager getDatabaseManager() {
		return databaseManager;
	}

	/**
	 * Setter of the property <tt>databaseManager</tt>
	 * @param databaseManager  The databaseManager to set.
	 * @uml.property  name="databaseManager"
	 */
	public void setDatabaseManager(DatabaseManager databaseManager) {
		this.databaseManager = databaseManager;
	}
	
	public int getInteger(Object o) throws NotRightTypeException{
		if (o instanceof Long) {
			Long new_name = (Long) o;
			long l = new_name;
			return (int)l;
		}
		else if (o instanceof Integer) {
			Integer new_name = (Integer) o;
			return new_name;
		}	
		throw new NotRightTypeException();
	}
}
